package ex5;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Scanner;

public class Application {
	
	private Illustration illustration = new Illustration();
	private ArrayList<String> commands = new ArrayList<String>();
	
        private static String help = "";
		static{ 
			help += "Commands are:\n";
			help += "quit\n";
			help += "create circle color x y radius\n";
			help += "create line color x1 y1 x2 y2\n";
			help += "move color x y\n";
			help += "remove color\n";
			help += "save fileName\n";
			help += "open fileName\n";
		};

	/**
	 * Lit sur input les commandes et les exécute.
	 * Les commandes sont:
	 *  - help
	 *  - quit
	 *  - create circle color x y radius
	 *  - create line color x1 y1 x2 y2
	 *  - move color x y
	 *  - remove color
	 *  - save fileName
	 *  - open fileName
	 */
	public void start()
	{
		initIllustration();
		boolean stop = false;
		Scanner in = new Scanner(System.in);
		while(!stop)
		{
			System.out.println("Enter your command (help for help):");

			if(!in.hasNext()) // a file was opened, read, and executed
                        {
                            in = new Scanner(System.in); 
                        }

                        //give back the control to the standard input
			String line = in.nextLine();
			String[] array = line.split(" ");
			int length = array.length;
			if (length>0)
			{
				String command = array[0];
				
                                /* HINT: the command pattern may help */
				if("help".equals(command))	{
					printHelp();
					continue;
				} else if("create".equals(command) && length>2) {
					commands.add(line);
					String form = array[1];
					char color = array[2].charAt(0);
					if(form.equals("circle") && length == 6) {
						createCircleCommand(array, color);
					} else if(form.equals("line") && length == 7) {
						createLineCommand(array, color);						
					}
				} else if("move".equals(command) && length==4) {
					commands.add(line);
					moveCommand(array);
				} else if("remove".equals(command) && length==2){
					commands.add(line);
					removeCommand(array);
				} else if("save".equals(command) && length==2){
					String fileName = array[1];
					saveCommand(fileName);
				} else if("open".equals(command) && length==2){
					String fileName = array[1];
                                        /* FIXME: we should not change the input
                                         * stream, but instead have an
                                         * independent command parser
                                         *
                                         */
					in = openCommand(in, fileName);
				} else if("quit".equals(command)){
					stop = true;
					continue;
				}
				illustration.draw();
			}
		}
	}

	/**
	 * initialise l'illustration avec des ASCIINumber 
	 * représentant l'heure de départ.
	 */
	private void initIllustration() {
        Calendar calendar = Calendar.getInstance();
        int hour = calendar.get(Calendar.HOUR);
        int min = calendar.get(Calendar.MINUTE);
        
		illustration.addAsciiNumber(new ASCIINumber(hour/10, new Point(0,0)));
		illustration.addAsciiNumber(new ASCIINumber(hour%10, new Point(0,4)));
		illustration.addAsciiNumber(new ASCIINumber(min/10, new Point(0,8)));
		illustration.addAsciiNumber(new ASCIINumber(min%10, new Point(0,12)));
	}

        /**
         * Gère la commande <strong>open</strong>.
         *
         * @param in        Le flux d'entrée retourné si le fichier ne peut être ouvert
         * @param fileName  Le fichier à lire. Ce fichier doit contenir des
         *                  commandes.
         */
	private Scanner openCommand(Scanner in, String fileName) {
		try {
			return new Scanner(new File(fileName));
		} catch (FileNotFoundException e) {
			System.out.println("Error: file not found: " + fileName);
                        System.out.println("Falling back to the input stream");
		}
		return in;
	}

        /**
         * Gère la commande <strong>save</strong>.
         *
         * @param fileName  Le fichier dans lequel écrire les commandes. 
         */
	private void saveCommand(String fileName) {
		FileWriter writer = null;

		try {
                        /* open for each stored command, put it back in the file */
			writer = new FileWriter(fileName);

			for (String comm : commands) {
				writer.write(comm+"\n");
			}
		} catch (IOException e) {
			System.out.println("File error: "+e.getMessage());
		} finally {
			try {
				writer.close();
			} catch (IOException e1) {
				// ignore.
			}
		}
	}

        /**
         * Gère la commande <strong>remove</strong>.
         *
         * @param arguments Les arguments de la commande (remove color).
         */
	private void removeCommand(String[] arguments) {
		char color = arguments[1].charAt(0);
		Form form = illustration.getForm(color);

		if (null == form) {
			System.out.println("Error: no shape of color: "+color);
		} else {
			illustration.removeForm(form);
		}
	}

        /**
         * Gère la commande <strong>move</strong>.
         *
         * @param arguments Les arguments de la commande (move color x y).
         */
	private void moveCommand(String[] arguments) {
		char color = arguments[1].charAt(0);
		Form form = illustration.getForm(color);

		if (null == form) {
			System.out.println("Error: no shape of color: "+color);
		} else {
			try {
				int x = Integer.parseInt(arguments[2]);
				int y = Integer.parseInt(arguments[3]);
				form.move(x, y);
			} catch (NumberFormatException e) {
				System.out.println("Error: x and y must be integers");
			}
		}
	}

        /**
         * Gère la commande <strong>create line</strong>.
         *
         * @param arguments Les arguments de la commande (create line color x1 y1 x2 y2).
         * @param color     La valeur de l'argument color
         */
	private void createLineCommand(String[] arguments, char color) {
		try {
			int x1 = Integer.parseInt(arguments[3]);
			int y1 = Integer.parseInt(arguments[4]);
			int x2 = Integer.parseInt(arguments[5]);
			int y2 = Integer.parseInt(arguments[6]);

			Point firstPoint = new Point(x1,y1);
			Point secondPoint = new Point(x2,y2);

			illustration.addForm(new Line(firstPoint, color, secondPoint));
		} catch (NumberFormatException e) {
			System.out.println("Error: x1, y1, x2, y2 must be integers");
		}
	}

        /**
         * Gère la commande <strong>create line</strong>.
         *
         * @param arguments Les arguments de la commande (create line color x1 y1 x2 y2).
         * @param color     La valeur de l'argument color
         */
	private void createCircleCommand(String[] arguments, char color) {
		try {
			int x = Integer.parseInt(arguments[3]);
			int y = Integer.parseInt(arguments[4]);
			int radius = Integer.parseInt(arguments[5]);

			Point center = new Point(x,y);

			illustration.addForm(new Circle(center, color,radius));
		} catch (NumberFormatException e) {
			System.out.println("Error: x, y and radius must be integers");
		}
	}

        /**
         * Gère la commande <strong>help</strong>.
         */
	private void printHelp() {
		System.out.println(Application.help);
	}
	
	public static void main(String[] args) {
		new Application().start();
	}

}
